<?php
namespace Teknisa\MAN\Controllers;

use Teknisa\MAN\Services\ExpImpNfceService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\MAN\Utilities\Resposta;
use Teknisa\MAN\Utilities;
use \Datetime;
use Zeedhi\Framework\DTO\Response\Message;

use NFePHP\NFe\Common\Standardize;
use NFePHP\Common\Certificate;
use NFePHP\Common\Soap\SoapCurl;
use NFePHP\NFe\Make;
use NFePHP\NFe\Tools;


use Teknisa\MAN\Services\TransmissaoNfceNewService;
use Teknisa\MAN\Factories\TransmissaoNFCE;


class ExpImpNfceController {     
    
    private $expImpNfceService, $utilService, $entityManager, $nfeTools, $transmissaoService, $notaFiscal;
    private $url = "https://rm.teknisa.com/export-xml-nfce/";
    const VERSAO_LAYOUT = '4.00';
    
    
    public static $logMessages = array();
    
    public function __construct (Utilities\Util $utilService, ExpImpNfceService $expImpNfceService, $instanceManager, TransmissaoNfceNewService $transmissaoService, TransmissaoNFCE $notaFiscal){
        $this->expImpNfceService = $expImpNfceService;
        $this->utilService = $utilService;
        $this->instanceManager = $instanceManager;
        $this->transmissaoService = $transmissaoService;
        $this->notaFiscal = $notaFiscal;
    }
    
    public static function escreverLog ($data) {
        array_push(self::$logMessages, $data);
    }
    
    public static function escreverLogArray($array) {
        foreach ($array as $linha) {
            self::escreverLog($linha);
        }
    }
    
    public static function getLog () {
        return self::$logMessages;
    }
    
    public function buscar (Request\Filter $request, Response $response) {
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
        $parametros = $this->buildParameters($request);
        
        $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::BUSCAR_NOTAS, $parametros);
        if ($result->error())
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        else 
            $response->addDataSet(new DataSet('vnd04400_expimpxmlnfce', $result->getResult()));
    }
    
    public function buildParameters (Request\Filter $request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $params = array();
        $params[':ALLFIEDLS'] = 'T';
        foreach ($conditions as $filtro) {
            
            if (is_array($filtro['value'])) {
                
                // Especifico desse controller
                if ($filtro['columnName'] == "DTVENDA") {
                    if (!empty($filtro['value'][0]) && !empty($filtro['value'][1])) {
                        $params[':P_DTINI'] = $filtro['value'][0];
                        $params[':P_DTFIM'] = $filtro['value'][1];
                    }
                }
                
                if(strpos($filtro['columnName'], '|') && $filtro['operator'] === 'MAPPED_LIKE_ALL'){
                    $colunas = explode('|', $filtro['columnName']);
                    foreach($colunas as $k => $con){
                        $params[':ALLFIEDLS'] = $filtro['value'][0]['_ALL'];
                    }
                }else{
                    $params[':'.$filtro['columnName']] = implode("_", $filtro['value']);     
                }
            } else {
                $params[':'.$filtro['columnName']] = $filtro['value']; 
            }
        }
        
        // Específico desse controller
        $params = $this->verificarDatas($params);
        $params = $this->formataParametros($params);
        return $params;
    }
    
    public function verificarDatas($parametros) {
        
        if (empty($parametros[':P_DTINI']) || empty($parametros[':P_DTFIM'])) {
            $parametros[':P_DTINI'] = 
            $parametros[':P_DTFIM'] = date('d/m/Y');
        }
        unset($parametros[':DTVENDA']);
        return $parametros;
    }
    
    public function formataParametros($parametros){
        
        if(isset($parametros[':CDFILIAL'])){
            $parametros[':P_CDFILIAL'] = $parametros[':CDFILIAL'];
            unset($parametros[':CDFILIAL']);
        }else{
            $parametros[':P_CDFILIAL'] = '';
        }
        
        if($parametros[':P_CDFILIAL'] == ''){
            $parametros[':P_TCDFILIAL'] = 'T';
        }else{
            $parametros[':P_TCDFILIAL'] = '';
        }
        
        if(isset($parametros[':CDCAIXA'])){
            $parametros[':P_CDCAIXA'] = $parametros[':CDCAIXA'];
            unset($parametros[':CDCAIXA']);
        }else{
            $parametros[':P_CDCAIXA'] = '';
        }
        
        if($parametros[':P_CDCAIXA'] == ''){
            $parametros[':P_TCDCAIXA'] = 'T';
        }else{
            $parametros[':P_TCDCAIXA'] = '';
        }
        
        if(isset($parametros[':SITUACAO'])){
            $parametros[':P_IDSTATUSNFCE'] = $parametros[':SITUACAO'];
            unset($parametros[':SITUACAO']);
        }else{
            $parametros[':P_IDSTATUSNFCE'] = 'A_C';
        }
        
        if(isset($parametros[':NRNOTAFISCALCE'])){
            $parametros[':NRNOTAFISCALCE'] = str_pad($parametros[':NRNOTAFISCALCE'], 9, '0', STR_PAD_LEFT);
        }else{
            $parametros[':NRNOTAFISCALCE'] = 'T';
        }
        
        return $parametros;
    }
    
    public function diretorioXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        
        $arrayCurlFields = array(
            'diretorio' => date('Ymd').'-'.uniqid(rand(), false)
        );
        $response->addDataSet(new DataSet('resultado', $arrayCurlFields));
    }
    
    public function formatXML($notas, $HABDESTAG = 'N'){
        if(!empty($notas) && count($notas) > 0){
            foreach ($notas as $key => $value) {
                if(!empty($value['DSARQXMLNFCE'])){
                    
                    $xml = str_replace('<?xml version="1.0" encoding="UTF-8"?>', '', $value['DSARQXMLNFCE']);
                    $xml = str_replace('<?xml version="1.0"?>', '', $xml);
                    
                    $verificaXML = strpos( $xml, 'NFe');
                    $tagPos = '';
                    
                    //verifica o tipo de emissao do arquivo
                    if($value['IDTPEMISVEND'] == 'N' && $verificaXML !== false){
                        
                        $xmlRetorno = simplexml_load_string($xml);  
                        //validar XML que esta salvo
                        if(!empty($xmlRetorno)){
                            $arrayData2 = $this->xmlToArray($xmlRetorno);
                            //verifica se XML foi assinado
                            $verificaXMLSign = strpos($xml, 'Signature');
                            if($verificaXMLSign === false){
                                $xml = $this->signNFCe($value, $xml);
                                if(!$xml){
                                    //valida os dados do XML
                                     if(!$this->validaXMLSalvo($arrayData2)){
                                         //regera XML com os dados de venda atualizado
                                         $xml = $this->callRemontaXML($value);
                                         $xml = $this->signNFCe($value, $xml);
                                     }
                                }
                            }
                             
                            $tagPos = '</infNFe>';
                            $xmlRetorno = simplexml_load_string($xml);  
                            $arrayData2 = $this->xmlToArray($xmlRetorno);
                                
                            $arrayDataEnviNFe = array();
                            if(isset($arrayData2['enviNFe']) && $HABDESTAG == 'S'){
                                $arrayDataEnviNFe['enviNFe'] = $arrayData2['enviNFe'];
                                $arrayData['NFe'] = $arrayData2['enviNFe']['NFe'];
                            }else if(isset($arrayData2['enviNFe']) && $HABDESTAG == 'N'){
                                $arrayDataEnviNFe['enviNFe'] = '';
                                $arrayData['NFe'] = $arrayData2['enviNFe']['NFe'];
                                
                                //retira conforme orientacao do delphi => O xml deve ser gerado a partir da tag <NFe xmlns="http://www.portalfiscal.inf.br/nfe">
                                $posIn = strpos($xml, '<NFe');
                                $xmlGerado = substr($xml, $posIn);
                                $posFi = strpos($xmlGerado, '</enviNFe>');
                                $xmlGerado = substr($xmlGerado, 0, $posFi);
                                $xml = $xmlGerado;
                                
                            }else{
                                $arrayData = $arrayData2;
                            }
                            
                            if(!empty($value['DTHRPROTOCONFCE'])){
                                $date = DateTime::createFromFormat('d/m/Y H:i:s', $value['DTHRPROTOCONFCE']);
                                $dataFormatada1 = $date->format('Y-m-d');
                                $dataFormatada2 = $date->format('H:i:s');
                                $stFusoHr = '-03:00';
                                $dhRecbto = $dataFormatada1."T".$dataFormatada2.$stFusoHr;    
                            }else{
                                $dhRecbto = $arrayData['NFe']['infNFe']['ide']['dhEmi'];    
                            }
                                
                            $stProtNfe = '<protNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="'.$arrayData['NFe']['infNFe']['@versao'].'">';
                            $stProtNfe .= '<infProt Id="ID'.$value['NRPROTOCOLONFCE'].'">';
                            $stProtNfe .= '<tpAmb>'.$value['IDTPAMBNFCE'].'</tpAmb>';
                            $stProtNfe .= '<verAplic>'.$arrayData['NFe']['infNFe']['@versao'].'</verAplic>';
                            $stProtNfe .= '<chNFe>'.$value['NRACESSONFCE'].'</chNFe>';
                            $stProtNfe .= '<dhRecbto>'.$dhRecbto.'</dhRecbto>';
                            $stProtNfe .= '<nProt>'.$value['NRPROTOCOLONFCE'].'</nProt>';
                            
                            $stProtNfe .= '<digVal>'.$arrayData['NFe']['Signature']['SignedInfo']['Reference']['DigestValue'].'</digVal>';
                            
                           
                            $stProtNfe .= '<cStat>100</cStat>';
                            $stProtNfe .= '<xMotivo>Autorizado o uso da NF-e</xMotivo>';
                            $stProtNfe .= '</infProt></protNFe>';
                            
                            $stTextoXML = '';
                            $stTextoXML = '<?xml version="1.0" encoding="UTF-8"?><nfeProc versao="'.$arrayData['NFe']['infNFe']['@versao'].'" xmlns="http://www.portalfiscal.inf.br/nfe">';
                            $stTextoXML = $stTextoXML.$xml.$stProtNfe;
                            $stTextoXML = $stTextoXML.'</nfeProc>';
                            
                            
                            //mudancao pois trara na query principal
                            /*$parametros[':SGESTADO'] = $value['SGESTADO'];
                            $parametros[':IDSERVICO'] = ($value['IDTPEMISVEND'] == 'N') ? 'NFC' : '';
                            $parametros[':IDAMBIENTE'] = $value['IDTPAMBNFCE'];
                            
                            $urlChave = $this->expImpNfceService->executarQuery(ExpImpNfceService::BUSCAR_URLCHAVE, $parametros);
                            if(!empty($urlChave->getResult())){
                                $url = $urlChave->getResult();
                                $stUrlChave = '<urlChave>'.$url[0]['WSSITEUFQRCODENFC'].'</urlChave>';
                            }else{
                                $stUrlChave = '<urlChave> </urlChave>';
                            }*/
                            
                            if(!empty($value['WSSITEUFQRCODENFC'])){
                                $stUrlChave = '<urlChave>'.$value['WSSITEUFQRCODENFC'].'</urlChave>';
                            }else{
                                $stUrlChave = '<urlChave> </urlChave>';
                            }
                            
                            if(strpos($stTextoXML, '<infNFeSupl>') == FALSE){
                                $stQRCode = '<qrCode><![CDATA['.$value['DSQRCODENFCE'].']]></qrCode>';
                                $stQRCODE = '<infNFeSupl>'.$stQRCode.$stUrlChave.'</infNFeSupl>';
                                //$posIn = strripos($stTextoXML, $tagPos);
                                $posFi = strpos($stTextoXML, '<Signature');
                                $xmlQrCode = substr($stTextoXML, 0, $posFi). $stQRCODE. substr($stTextoXML, $posFi);
                                $stTextoXML = $xmlQrCode;
                            }
                            
                            $notas[$key]['DSARQXMLNFCE'] = $stTextoXML;
                            $notas[$key]['NOMEDSARQXMLNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-rec.xml';
                            
                            if($value['IDSTATUSNFCE'] == 'C'){
                                
                                $xmlCancSalvo = $value['DSARQXMLCANCNFCE'];
                                $xmlCancelamentoSalvo = simplexml_load_string($xmlCancSalvo);
                                
                                $xmlCanc = '<procEventoNFe></procEventoNFe>';
                                $xmlCancelamento = simplexml_load_string($xmlCanc);
                                
                                $xmlCancelamento->addAttribute('versao', '1.00');
                                $xmlCancelamento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
                                
                                if(!empty($value['DTHRPROTOCOCANC'])){
                                    $dateCan = DateTime::createFromFormat('d/m/Y H:i:s', $value['DTHRPROTOCOCANC']);
                                    $dataFormatada1Can = $dateCan->format('Y-m-d');
                                    $dataFormatada2Can = $dateCan->format('H:i:s');
                                    $stFusoHrCan = '-03:00';
                                    $dhRecbtoCan = $dataFormatada1Can."T".$dataFormatada2Can.$stFusoHrCan;    
                                }else{
                                    $dataFormatada1Can = date('Y-m-d');
                                    $dataFormatada2Can = date('H:i:s');
                                    $stFusoHrCan = '-03:00';
                                    $dhRecbtoCan = $dataFormatada1Can."T".$dataFormatada2Can.$stFusoHrCan;    
                                }
                                
                                if(isset($xmlCancelamentoSalvo->evento)){
                                    $evento = $xmlCancelamento->addChild('evento');
                                    $evento->addAttribute('versao', '1.00');
                                    $evento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
                                    if($xmlCancelamentoSalvo->evento->infEvento){
                                        $infEvento = $evento->addChild('infEvento');
                                        
                                        $arrayID = $xmlCancelamentoSalvo->evento->infEvento->attributes()->Id;
                                        foreach((array)$arrayID as $key => $val){
                                            $id = $val;
                                        }
                                        
                                        $infEvento->addAttribute('Id', $id);
                                        $infEvento->addChild('cOrgao',$xmlCancelamentoSalvo->evento->infEvento->cOrgao);
                                        $infEvento->addChild('tpAmb',$xmlCancelamentoSalvo->evento->infEvento->tpAmb);
                                        $infEvento->addChild('CNPJ',$xmlCancelamentoSalvo->evento->infEvento->CNPJ);
                                        $infEvento->addChild('chNFe',$xmlCancelamentoSalvo->evento->infEvento->chNFe);
                                        $infEvento->addChild('dhEvento',$xmlCancelamentoSalvo->evento->infEvento->dhEvento);
                                        $infEvento->addChild('tpEvento',$xmlCancelamentoSalvo->evento->infEvento->tpEvento);
                                        $infEvento->addChild('nSeqEvento',$xmlCancelamentoSalvo->evento->infEvento->nSeqEvento);
                                        $infEvento->addChild('verEvento',$xmlCancelamentoSalvo->evento->infEvento->verEvento);
                                        
                                        if($xmlCancelamentoSalvo->evento->infEvento->detEvento){
                                            $detEvento = $infEvento->addChild('detEvento');
                                            $arrayID = $xmlCancelamentoSalvo->evento->infEvento->detEvento->attributes()->versao;
                                            foreach((array)$arrayID as $key => $val2){
                                                $versao = $val2;
                                            }
                                            $detEvento->addAttribute('versao', $versao);
                                            $detEvento->addChild('descEvento',$xmlCancelamentoSalvo->evento->infEvento->detEvento->descEvento);
                                            $detEvento->addChild('nProt',$xmlCancelamentoSalvo->evento->infEvento->detEvento->nProt);
                                            $detEvento->addChild('xJust',$xmlCancelamentoSalvo->evento->infEvento->detEvento->xJust);    
                                        }else{
                                            $detEvento = $infEvento->addChild('detEvento');
                                            $detEvento->addAttribute('versao', '1.00');
                                            $detEvento->addChild('descEvento','Cancelamento');
                                            $detEvento->addChild('nProt',$value['NRPROTOCOLONFCE']);
                                            $detEvento->addChild('xJust',$value['DSRAZAOCANCNFCE']);    
                                        }
                                    }else{
                                        $infEvento = $evento->addChild('infEvento');
                                        $infEvento->addAttribute('Id', 'ID'.'110111'.$value['NRACESSONFCE'].'01');
                                        $infEvento->addChild('cOrgao',$arrayData['NFe']['infNFe']['ide']['cUF']);
                                        $infEvento->addChild('tpAmb',$value['IDTPAMBNFCE']);
                                        $infEvento->addChild('CNPJ',$value['NRINSJURIEST']);
                                        $infEvento->addChild('chNFe',$value['NRACESSONFCE']);
                                        
                                        $infEvento->addChild('dhEvento',$dhRecbtoCan);
                                        $infEvento->addChild('tpEvento','110111');
                                        $infEvento->addChild('nSeqEvento','1');
                                        $infEvento->addChild('verEvento','1.00');
                                        
                                        $detEvento = $infEvento->addChild('detEvento');
                                        $detEvento->addAttribute('versao', '1.00');
                                        $detEvento->addChild('descEvento','Cancelamento');
                                        $detEvento->addChild('nProt',$value['NRPROTOCOLONFCE']);
                                        $detEvento->addChild('xJust',$value['DSRAZAOCANCNFCE']);
                                    }
                                }else{
                                    $evento = $xmlCancelamento->addChild('evento');
                                    $evento->addAttribute('versao', '1.00');
                                    $evento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
                                    
                                    $infEvento = $evento->addChild('infEvento');
                                    $infEvento->addAttribute('Id', 'ID'.'110111'.$value['NRACESSONFCE'].'01');
                                    $infEvento->addChild('cOrgao',$arrayData['NFe']['infNFe']['ide']['cUF']);
                                    $infEvento->addChild('tpAmb',$value['IDTPAMBNFCE']);
                                    $infEvento->addChild('CNPJ',$value['NRINSJURIEST']);
                                    $infEvento->addChild('chNFe',$value['NRACESSONFCE']);
                                    
                                    $infEvento->addChild('dhEvento',$dhRecbtoCan);
                                    $infEvento->addChild('tpEvento','110111');
                                    $infEvento->addChild('nSeqEvento','1');
                                    $infEvento->addChild('verEvento','1.00');
                                    
                                    $detEvento = $infEvento->addChild('detEvento');
                                    $detEvento->addAttribute('versao', '1.00');
                                    $detEvento->addChild('descEvento','Cancelamento');
                                    $detEvento->addChild('nProt',$value['NRPROTOCOLONFCE']);
                                    $detEvento->addChild('xJust',$value['DSRAZAOCANCNFCE']);
                                }
                                
                                if(isset($xmlCancelamentoSalvo->retEnvEvento)){
                                    if(isset($xmlCancelamentoSalvo->retEnvEvento->retEvento)){
                                        $retEvento = $xmlCancelamento->addChild('retEvento');
                                        $retEvento->addAttribute('versao', '1.00');
                                        $retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
                                        if(isset($xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento)){
                                            $infEventoRet = $retEvento->addChild('infEvento');
                                            $infEventoRet->addChild('tpAmb',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->tpAmb);
                                            $infEventoRet->addChild('verAplic',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->verAplic);
                                            $infEventoRet->addChild('cOrgao',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->cOrgao);
                                            $infEventoRet->addChild('cStat',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->cStat);
                                            $infEventoRet->addChild('xMotivo',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->xMotivo);
                                            $infEventoRet->addChild('chNFe',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->chNFe); 
                                            $infEventoRet->addChild('tpEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->tpEvento);
                                            $infEventoRet->addChild('xEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->xEvento);
                                            $infEventoRet->addChild('nSeqEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->nSeqEvento);
                                            $infEventoRet->addChild('dhRegEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->dhRegEvento);
                                            $infEventoRet->addChild('nProt',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->nProt);
                                        }else{
                                            $infEventoRet = $retEvento->addChild('infEvento');
                                            $infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
                                            $infEventoRet->addChild('verAplic','SEFAZ');
                                            $infEventoRet->addChild('cOrgao',$arrayData['NFe']['infNFe']['ide']['cUF']);
                                            $infEventoRet->addChild('cStat','135');
                                            $infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
                                            $infEventoRet->addChild('chNFe',$value['NRACESSONFCE']);
                                            $infEventoRet->addChild('tpEvento','110111');
                                            $infEventoRet->addChild('xEvento','Cancelamento');
                                            $infEventoRet->addChild('nSeqEvento','1');
                                            $infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
                                            $infEventoRet->addChild('nProt',$value['NRPROTOCOLOCANC']);
                                        }
                                    }else{
                                        $retEvento = $xmlCancelamento->addChild('retEvento');
                                        $retEvento->addAttribute('versao', '1.00');
                                        $retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
                                        $infEventoRet = $retEvento->addChild('infEvento');
                                        $infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
                                        $infEventoRet->addChild('verAplic','SEFAZ');
                                        $infEventoRet->addChild('cOrgao',$arrayData['NFe']['infNFe']['ide']['cUF']);
                                        $infEventoRet->addChild('cStat','135');
                                        $infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
                                        $infEventoRet->addChild('chNFe',$value['NRACESSONFCE']);
                                        $infEventoRet->addChild('tpEvento','110111');
                                        $infEventoRet->addChild('xEvento','Cancelamento');
                                        $infEventoRet->addChild('nSeqEvento','1');
                                        $infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
                                        $infEventoRet->addChild('nProt',$value['NRPROTOCOLOCANC']);
                                    }
                                }else{
                                    $retEvento = $xmlCancelamento->addChild('retEvento');
                                    $retEvento->addAttribute('versao', '1.00');
                                    $retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
                                    $infEventoRet = $retEvento->addChild('infEvento');
                                    $infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
                                    $infEventoRet->addChild('verAplic','SEFAZ');
                                    $infEventoRet->addChild('cOrgao',$arrayData['NFe']['infNFe']['ide']['cUF']);
                                    $infEventoRet->addChild('cStat','135');
                                    $infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
                                    $infEventoRet->addChild('chNFe',$value['NRACESSONFCE']);
                                    $infEventoRet->addChild('tpEvento','110111');
                                    $infEventoRet->addChild('xEvento','Cancelamento');
                                    $infEventoRet->addChild('nSeqEvento','1');
                                    $infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
                                    $infEventoRet->addChild('nProt',$value['NRPROTOCOLOCANC']);
                                }
                                
                                $XCanc = $xmlCancelamento->asXML();
                                $notas[$key]['DSARQXMLCANCNFCE'] = $XCanc;
                                
                                $notas[$key]['NOMEDSARQXMLCANCNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-can.xml';
                            }    
                             
                             
                        }
                        
                    }else{
                        //ver o que faz com essa parte ["infNFe"]
                        $notas[$key]['NOMEDSARQXMLNFCE'] = 'AD'.$value['NRACESSONFCE'].'.xml';
                        if($value['IDSTATUSNFCE'] == 'C'){
                            $notas[$key]['NOMEDSARQXMLCANCNFCE'] = 'ADC'.$value['NRACESSONFCE'].'.xml';
                        }
                    }
                }
            }
        }
        return $notas;
    }
    
    public function exportarXml(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        //tratamento para habilitar ou desabilitar tags <idLote> e <indSinc>
        if(!isset($row['HABDESTAG'])){
            $row['HABDESTAG'] = 'N';
        }
        //realizar tratamento dos arquivos XMLs
        $notas = $this->formatXML($row['arrRows'], $row['HABDESTAG']);
        
        $arrayCurlFields = array(
            'script' => 'EXP',
            'diretorio' => $row['diretorio'],
            'rows' => $notas
        );
        
        $fields_string = http_build_query($arrayCurlFields);
        if(!empty($fields_string)){ 
           try {
            
                $ch = curl_init();
                curl_setopt( $ch, CURLOPT_URL, $this->url );
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch,CURLOPT_POST, 1);
                curl_setopt($ch,CURLOPT_POSTFIELDS, $fields_string);
                $retorno = json_decode(curl_exec( $ch ));
                $array = (array) $retorno;
                curl_close( $ch );
                
                $response->addDataSet(new DataSet('resultado', $array));
                
            }
            catch(\Exception $e){
                throw new Exception("URL inválida",0,$e);
            }     
        }
        
    }
    
    public function compactarXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        
        $arrayCurlFields = array( 
            'diretorio' => $row['diretorio'],
            'script' => 'COMP'
        );
        
        try {
            
            $fields_string = http_build_query($arrayCurlFields);
            $ch = curl_init();
            curl_setopt( $ch, CURLOPT_URL, $this->url );
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
            $retorno = json_decode(curl_exec( $ch ));
            $array = (array) $retorno;
            curl_close( $ch );
        }
        catch(\Exception $e){
            throw new Exception("URL inválida",0,$e);
        }
        
        $response->addDataSet(new DataSet('resultado', $array));
    }
    
    public function deleteArquivosXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        
        $arrayCurlFields = array( 
            'diretorio' => $row['diretorio'].'.zip',
            'script' => 'DEL'
        );
        
        try {
            
            $fields_string = http_build_query($arrayCurlFields);
            $ch = curl_init();
            curl_setopt( $ch, CURLOPT_URL, $this->url );
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
            $retorno = json_decode(curl_exec( $ch ));
            $array = (array) $retorno;
            curl_close( $ch ); 
        }
        catch(\Exception $e){
            throw new Exception("URL inválida",0,$e);
        }
        
        $response->addDataSet(new DataSet('resultado', $array));
    }
    
    private function xmlToArray($xml, $options = array()) {
        $defaults = array(
            'namespaceSeparator' => ':', // você pode querer que isso seja algo diferente de um cólon
            'attributePrefix' => '@',    // para distinguir entre os nós e os atributos com o mesmo nome
            'alwaysArray' => array(),    // array de tags que devem sempre ser array
            'autoArray' => true,         // só criar arrays para as tags que aparecem mais de uma vez
            'textContent' => '$',        // chave utilizada para o conteúdo do texto de elementos
            'autoText' => true,          // pular chave "textContent" se o nó não tem atributos ou nós filho
            'keySearch' => false,        // pesquisa opcional e substituir na tag e nomes de atributos
            'keyReplace' => false        // substituir valores por valores acima de busca
        );
        $options = array_merge($defaults, $options);
        $namespaces = $xml->getDocNamespaces();
        $namespaces[''] = null; // adiciona namespace base(vazio) 
    
        // Obtém os atributos de todos os namespaces
        $attributesArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->attributes($namespace) as $attributeName => $attribute) {
                // Substituir caracteres no nome do atributo
                if ($options['keySearch']) $attributeName =
                        str_replace($options['keySearch'], $options['keyReplace'], $attributeName);
                $attributeKey = $options['attributePrefix']
                        . ($prefix ? $prefix . $options['namespaceSeparator'] : '')
                        . $attributeName;
                $attributesArray[$attributeKey] = (string)$attribute;
            }
        }
    
        // Obtém nós filhos de todos os namespaces
        $tagsArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->children($namespace) as $childXml) {
                // Recursividade em nós filho
                $childArray = $this->xmlToArray($childXml, $options);
                reset($childArray);
                list($childTagName, $childProperties) = $this->myEach($childArray);
                // Substituir caracteres no nome da tag
                if ($options['keySearch']) $childTagName =
                        str_replace($options['keySearch'], $options['keyReplace'], $childTagName);
                // Adiciona um prefixo namespace, se houver
                if ($prefix) $childTagName = $prefix . $options['namespaceSeparator'] . $childTagName;
    
                if (!isset($tagsArray[$childTagName])) {
                    // Só entra com esta chave
                    // Testa se as tags deste tipo deve ser sempre matrizes, não importa a contagem de elementos
                    $tagsArray[$childTagName] =
                            in_array($childTagName, $options['alwaysArray']) || !$options['autoArray']
                            ? array($childProperties) : $childProperties;
                } elseif (
                    is_array($tagsArray[$childTagName]) && array_keys($tagsArray[$childTagName])
                    === range(0, count($tagsArray[$childTagName]) - 1)
                ) {
                    $tagsArray[$childTagName][] = $childProperties;
                } else {
                    $tagsArray[$childTagName] = array($tagsArray[$childTagName], $childProperties);
                }
            }
        }
    
        // Obtém o texto do nó
        $textContentArray = array();
        $plainText = trim((string)$xml);
        if ($plainText !== '') $textContentArray[$options['textContent']] = $plainText;
    
        $propertiesArray = !$options['autoText'] || $attributesArray || $tagsArray || ($plainText === '')
                ? array_merge($attributesArray, $tagsArray, $textContentArray) : $plainText;
    
        // Retorna o nó como array
        return array(
            $xml->getName() => $propertiesArray
        );
    }
    
    private function myEach(&$arr) {
        $key = key($arr);
        $result = ($key === null) ? false : [$key, current($arr), 'key' => $key, 'value' => current($arr)];
        next($arr);
        return $result;
    }
    
    private function signNFCe($nota, $unsignedXML){
        
		//le certificado
		$certificado = $this->getCertificado($nota);
		//config certificado
		$this->nfeTools = $this->transmissaoService->getNfeTools($nota, $certificado);
		
    	try {
    		
    		$XmlSign = $this->nfeTools->signNFe($unsignedXML);
		    if(!empty($XmlSign)){
				return $XmlSign;
		    }else{
		    	return false;
		    }
			
		} catch (\Exception $e) {
			return false;
		}
    }
    
	
	private function getCertificado($nota){
		
		$params[':CDINSCESTA'] = $nota['CDINSCESTA'];
		$params[':CDFILIAL'] = $nota['CDFILIAL'];
		
		$result = $this->expImpNfceService->executarQuery(TransmissaoNFCE::GET_LINK_CERTIFICADO, $params);
		if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $result = $result->getResult();
        }
        
		if (sizeof($result) > 0) {
			if (empty($result[0]['CERTIFICADO'])) {
				throw new \Exception('Erro, certificado não parametrizado. Verifique os parâmetros nas seguintes telas: Empresa -> Inscrição Estadual, e Parâmetros da Unidade -> Frente de Caixa -> Definições NFC-e / SAT!');
			}
			$link = $result[0]['DSDIRCERTIFICADO'];
			
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $link);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_TIMEOUT, 5);
			
			$resultLink = curl_exec($ch);
            $err    = curl_errno($ch);
            $errmsg = curl_error($ch);
			curl_close($ch);
			
			if ($err){
                throw new \Exception("Erro ao obter conteúdo do Certificado Digital: " . $errmsg);
            } 
            
            if (strpos($resultLink, 'Not Found')) {
            	throw new \Exception("Erro 404. Não foi possível recuperar o certificado no servidor.". PHP_EOL . 
            	"Verificar se o Link do Servidor de Imagens encontra-se parametrizado em Paramêtros Gerais => Gerais => Geral." . PHP_EOL . 
            	"Verificar se o certificado foi parametrizado em Parâmetros da Unidade => Frente de Caixa => Definições NFC-e/SAT e Empresa => Inscrição Estadual" . PHP_EOL .$errmsg);
            }
            
		} else {
			throw new \Exception('Não foi possível recuperar os dados de Inscrição Estadual parametrizada.' . PHP_EOL . 
            	'Verificar se o certificado foi parametrizado em Parâmetros da Unidade => Frente de Caixa => Definições NFC0e/SAT e Empresa => Inscrição Estadual');
		}
		return $result;
	}
    
    private function validaXMLSalvo($arrayXML){
        
        $valorControle = true;
        foreach($arrayXML as $key => $value){
            //tag NFe
            if(!isset($value['NFe'])){
                $valorControle = false;
            }
            
            //tag infNFe
            if(!isset($value['NFe']['infNFe'])){
                $valorControle = false;
            }
            
            //tag versao
            if(!empty($value['NFe']['infNFe']['@versao'])){
                 $valorControle = false;
            }
            
            //tag IDE
            $ide = $value['infNFe']['ide'];
            if(!empty($ide)){
                foreach($ide as $k => $v){
                     if(!empty($v['cUF'])){
                         $valorControle = false;
                     }
                     if(!empty($v['cUF'])){
                         $valorControle = false;
                     }
                     if(!empty($v['serie'])){
                         $valorControle = false;
                     }
                     if(!empty($v['nNF'])){
                         $valorControle = false;
                     }
                     if(!empty($v['cMunFG'])){
                         $valorControle = false;
                     }
                     if(!empty($v['cDV'])){
                         $valorControle = false;
                     }
                     if(!empty($v['tpAmb'])){
                         $valorControle = false;
                     }
                }   
            }else{
                 $valorControle = false;
            }
            
            //tag IDE
            $emit = $value['infNFe']['emit'];
            if(!empty($emit)){
                foreach($emit as $k => $v){
                     if(!empty($v['CNPJ'])){
                         $valorControle = false;
                     }
                     if(!empty($v['xNome'])){
                         $valorControle = false;
                     }
                     if(!empty($v['xFant'])){
                         $valorControle = false;
                     }
                     if(!empty($v['IE'])){
                         $valorControle = false;
                     }
                     if(!empty($v['CNAE'])){
                         $valorControle = false;
                     }
                     if(!empty($v['CRT'])){
                         $valorControle = false;
                     }
                }   
            }else{
                 $valorControle = false;
            }
        }
        
        return $valorControle;
        
    }
    
    private function callRemontaXML($nota){
	    
        $parametros = array(
            ':CDFILIAL' => $nota['CDFILIAL'],
            ':CDCAIXA' => $nota['CDCAIXA'],
            ':CDLOJA' => $nota['CDLOJA'],
            ':NRSEQVENDA'    => $nota['NRSEQVENDA']
        );
        
        $result = $this->expImpNfceService->executarQuery(TransmissaoNFCE::ADQTRANSMISSAONFCE, $parametros);
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            if(!empty($result->getResult())){
               $resposta = $result->getResult();
               $resposta[0]['CDOPERADOR'] = null;
	    	   $resposta[0]['TOTALVENDA'] = $nota['VRTOTALVENDA'];
	    	   $resposta[0]['DTABERTUR'] = $nota['DTABERTUR'];
               $retornoXML = $this->transmissaoService->montaXMLTransmitir($resposta, false);
               
               if(!$retornoXML['error']){
                   return $retornoXML['xml'];
               }
            }
        }
    }
    
    const DEFAULT_XML = '
        <NFe xmlns="http://www.portalfiscal.inf.br/nfe">
        </NFe>';
}